function [Z, X, Y, alpha, beta, Y3, Z_r, X_r, Y_r, alpha_r, Y3_r, sigma2, sigma2_r]=gendata(G, g, I, J);
% gendata: generate original data
%
% Input:
% G: number of genes
% I: number of repeation
% 
% Output:
% alpha:        - G x 1 matrix, gene expression level                    
% Z:            - Location, (G*I) x 12 matrix
%                 first 4 columns refer the location (row), 
%                       one of these 4 columns is 1, the rest three are 0;
%                 last 8 columns refer the location (column)
%                       one of these 8 columns is 1, the rest seven are 0;
%                 [ Z_row (G*I, 4)  Z_column (G*I, 8) ]
% X:            - (G*I) x 1 matrix, ratio of log-intensity of Cy5 over Cy3
% Y:            - (G*I) x 1 matrix, average of log-intensity of Cy5 and Cy3
% 
% Notes:
%       all the vectors are recorded in the following way:
%       e.g  X' = (x_{1, 1}, x_{1, 2}, ... , x_{1, I), x_{2, 1}, ..., x_{2, I},
%                 ..., x_{G, 1}, ... x_{G, I} ) 
%                 
%       Copyright (c) T. Huang 2003
%

% generate alpha, let alpha be double exponential
% double exponential == difference of two independent exponential 
alpha = zeros(G+g,1);
alpha1a = exprnd(1, 250, 2);
alpha1 = (alpha1a(:, 1) - alpha1a(:, 2));
alpha(G-149:G+100)=alpha1;

alpha_r=alpha(1:G);

% generate beta, record Z
beta=zeros(48*J,1);                          %48 blocks
for i=1:J;
    beta_blk=[-0.15	 -0.3	-0.15	0.01	-0.01	0.04	0.07	0.08	0.08	0.23	0.22	0.09
               0.11	 0.24	0.28	0.24	0.07	0.22	0.19	0.15	-0.03	-0.05	-0.02	0.03
              -0.14	 -0.04	-0.18	0.06	-0.05	-0.02	-0.17	-0.02	0.08	0.08	-0.04	-0.16
               0.04	 0.01	-0.05	-0.12	0.07	-0.19	-0.03	-0.15	-0.07	-0.22	-0.11	-0.22]';
    beta(48*(i-1)+1:48*i) = beta_blk;         %used to be no mutiplier 10
end;

Z_G = zeros(G*I, 48);            %the random location of replicated genes

for i = 1:(G*I); 
    Z_G(i, ceil(rand(1, 1)*48)) = 1;    
end;


Z_g = zeros(g, 48);            % the random locations of non-replicated genes

for i=1: g; 
     Z_g(i, ceil(rand(1, 1)*48)) = 1;     
end;


Z_blk = [ Z_G ; Z_g];

Z=kron( diag(ones(J,1)),Z_blk);

Z_r=kron( diag(ones(J,1)), Z_G);


% generate X, mixed uniform
for i = 1:((G*I+g)*J);
    if rand(1,1) < .2;
         X(i, 1) = 4 + rand(1, 1)*12;
    else             
        X(i, 1) = 16 - 10*rand(1, 1).^(1/4);
    end;
end;

X_r=X(1:G*I);
for i = 1:J-1
    X_r=[X_r; X(i*(G*I+g)+1:i*(G*I+g)+G*I)];
end
    

% calculate Y
Y1=zeros((G*I+g)*J,1);              %gene effect
for i=1:J;
    Y1((i-1)*(G*I+g)+1:i*(G*I+g))= [ (reshape(ones(I, 1)*(alpha(1:G))', G*I, 1)); (alpha(G+1:g+G))] ;
end;

Y1_r=zeros(G*I*J,1);
for i=1:J;
    Y1_r((i-1)*G*I+1:i*(G*I))= [ (reshape(ones(I, 1)*(alpha(1:G))', G*I, 1))']' ;
end;

Y2 = Z*beta;                        %block effect
Y2_r=Z_r*beta;
          
Y3=(.3592-sqrt((X-4)/32))*5;        %intensity effect  
Y3_r = (.3592-sqrt((X_r-4)/32))*5;


tmpx = reshape(X,I,G*J);
tmpmean = mean (tmpx,1);
X22=X;
for i=1:(G*J);
    X22(i*I-2:i*I)=tmpmean(i);
end;
sigma2 = 0.15 + 0.015 *(X22-9).^2;      
sigma2(X22<9) = 0.15; 

sigma2_r=sigma2(1:G*I);
for i=1:J-1;
    sigma2_r=[sigma2_r' sigma2(i*(G*I+g)+1:i*(G*I+g)+G*I)']';
end;

error = sqrt(sigma2).*randn((G*I+g)*J, 1);
error_r=error(1:G*I);
for i=1:J-1;
    error_r=[error_r' error(i*(G*I+g)+1:i*(G*I+g)+G*I)']';
end;


Y = Y1 + Y2 + Y3 + error;
Y_r=Y1_r + Y2_r + Y3_r + error_r;  
